/*
-----------------------------------------------------------------------------------------
***   Automatické vypnutí CNC stroje po dokončení zadaného úkonu s LCD displejem      ***
***   lze využít na strojích např. 3D tiskárna, fréza apod.                           ***
***   Verze 1.00        Arduino IDE v1,65   ATmega168/328-Arduino nano,mini,ATtiny85  ***
***   Petr Peterka                          Datum: 15/12/2019                         ***
-----------------------------------------------------------------------------------------
Projekt zabírá 2 128 bytů (6%)  úložného místa pro program. Maximum je 30 720 bytů.

Program realizující automatické vypnutí CNC stroje po dokončení zadaného úkonu.
Jedná se o externí doplněk pro každý CNC stroj, který touto funkcí nedisponuje již v základním vybavení.
Tímto externím zařízením může být dovybavena každá 3D tiskárna, CNC fréza apod.
Pracuje na principu vypnutí hlavního síťového napájení do CNC zařízení po dokončení úkonu.
Po zapnutí napájení a provedení resetu se okamžitě sepne výstupní relé, jehož kontakt
překlenuje hlavní vypínač. Tím se udrží hlavní síťový přívod do CNC stroje. Pokud je potřeba
využití funkce automatického vypnutí po provedení úkonu, je možné vypnout hlavní vypínač.
Není-li potřeba použít funkci automatického vypnutí, je možné ponechat hlavní vypínač zapnutý.
Je-li potřeba CNC zařízení vypnout dříve, než uplyne nastavený čas vypnutí, je možné pomocí
stisku tlačítka vypnout relé. Tlačítko má více funkcí. Poté je možné vypnout mechanický hlavní
vypínač CNC stroje.

Popis funkce programu:
Po provedení resetu se sepne výstupní pin na relé. Dále se načtou stavy jumperů pro nastavení
času pro zpožděné vypnutí. Možné jsou čtyři druhy nastavení času vypnutí a to 5min, 10min, 15min a 20min.
Nastavení délky časovky zobrazí indikační LED počtem bliknutí 1 až 4, přičemž 1x bliknutí odpovídá
nastavené časovce 5min, pak 2x = 10min, 3x = 15min a 4x = 20min. Nastavení časovky pomocí jumperů je popsáno níže. 
Po zobrazení nastavené časovky zařízení přejde do klidového stavu, kdy čeká na aktivaci od externího snímače
(infrazávory), nebo tlačítka. Klidový stav zařízení je indikován bliknutím LED na dobu 50ms po 2s.

Automatický režim:
Dojde-li k aktivaci zařízení od externího snímače (infrazávory), začne se odpočítávat nastavený čas pro
zpožděné vypnutí. Při aktivaci externího snímače se LED rozsvítí. Stav, kdy je odpočítáván čas pro
zpožděné vypnutí relé, je indikován blikáním LED se střídou 50% a periodou 4s (2s svit, 2s zhasnuta).
Po dokončení odpočtu času je vypnuto relé. LED se rozbliká se střídou 50% a periodou 100ms (50ms svit).

Manuální režim:
Krátkým stiskem tlačítka (<3s) dojde k resetu a uvedení celého zařízení do počátečního stavu.
Tedy dojde k opětovnému sepnutí relé, nastavení časovky pro zpožděné vypnutí.
Dlouhým stiskem tlačítka (>3s) dojde k okamžitému vypnutí relé a rozblikání LED s 50ms časovkou.

Tlačítko funkce:
Krátký stisk (<3s)- reset zařízení a uvedení do počátečního stavu
Dlouhý stisk (>3s) - manuální vypnutí relé a tím i možné vypnutí tiskárny
Stisk tlačítka je indikován zhasnutím LED.

LED1 indikace funkce:
Blikání pomalé nepravidelné - 50ms svit a 2000ms zhasnuta - zařízení je v klidovém stavu, relé zapnuto a připraveno
Blikání pomalé pravidelné - 2000ms svit a 2000ms zhasnuta - zařízení aktivováno a časovka vypnutí je odpočítávána
Blikání rychlé pravidelné - 50ms svit a 50ms zhasnuta - zařízení deaktivováno, časovka vypnutí dokončena a relé vypnuto
Rozsvícena trvale - aktivace snímače INFRA závory
Zhasnuta trvale - aktivace stisku tlačítka

Nastavené hodnoty času zpožděného vypnutí pomocí nastavovacích jumperů - propojek:
Aktivace = propojka zkratována - elektricky propojena.
J1 - aktivace, J2 - aktivace = nastavený čas 5min
J1 - aktivace, J2 - deaktivace = nastavený čas 10min
J1 - deaktivace, J2 - aktivace = nastavený čas 15min
J1 - deaktivace, J2 - deaktivace = nastavený čas 20min
*/

//-----------------------------------------------------------------------------------------
//***   Definice IO pinu periferií pro ATtiny85 modul                                   ***
//-----------------------------------------------------------------------------------------
#define RELE_PIN 5                     //Výstup na relé, pin P5, log.0 relé sepnuto 
#define LED1_PIN 4                     //Led zelená pro indikaci stavu zařízení, pin P4
#define SWITCH_PIN 3                   //Tlačítko pro manuální vypnutí/zapnutí relé, pin P3
#define J2_PIN 2                       //Jumper 2 pro nastavení délky časovky, pin P2
#define J1_PIN 1                       //Jumper 1 pro nastavení délky časovky, pin P1
#define INFRA_PIN 0                    //Snímač - infra závora, pin P0


//-----------------------------------------------------------------------------------------
//***   Definice logické úrovně aktivace IO pinů, 1 = aktivace v log.1 a opačně         ***
//-----------------------------------------------------------------------------------------
#define RELE_AKTIV      0               //Stav pinu při aktivaci výstupního relé
#define LED1_AKTIV      1               //Stav pinu při aktivaci - rozsvícení LED1
#define J1_PIN_AKTIV    0               //Stav pinu při aktivaci pro Jumper 1
#define J2_PIN_AKTIV    0               //Stav pinu při aktivaci pro Jumper 2
#define SWITCH_AKTIV    0               //Stav pinu při aktivaci tlačítka
#define INFRA_AKTIV     1               //Stav pinu při aktivaci infra snímače


//-----------------------------------------------------------------------------------------
//***   Definice konstant                                                               ***
//-----------------------------------------------------------------------------------------
#define CAS_5min        300000          //Hodnota času [ms] pro délku vypnutí - 5min = 300000ms(300s)        
#define CAS_10min       600000          //Hodnota času [ms] pro délku vypnutí - 10min = 600000ms(600s) 
#define CAS_15min       900000          //Hodnota času [ms] pro délku vypnutí - 15min = 900000ms(900s)  
#define CAS_20min       1200000         //Hodnota času [ms] pro délku vypnutí - 20min = 1200000ms(1200s)    
#define CAS_BLIK1_LED1  50              //Hodnota času [ms] pro délku blikání LED1 - 50ms
#define CAS_BLIK2_LED1  2000            //Hodnota času [ms] pro délku blikání LED1 - 2000ms(2s)
#define CAS_STISK_LONG  3000            //Hodnota času [ms] pro délku dlouhého stisku tlačítka - 3000ms(3s)
        

//-----------------------------------------------------------------------------------------
//***   Deklarace globálních proměnných                                                 ***
//-----------------------------------------------------------------------------------------
boolean       SWITCH_PIN_State = 0;     //Bit udávající stav tlačítka, 0 = nestisknuto, 1 = stisknuto
boolean       SWITCH_Stisk = 0;         //Bit udávající stisk tlačítka, 0 = nestisknuto, 1 = stisknuto 
boolean       J1_PIN_State = 0;         //Bit udávající stav jumperu J1, 0 = propojeno, 1 = nepropojeno
boolean       J2_PIN_State = 0;         //Bit udávající stav jumperu J2, 0 = propojeno, 1 = nepropojeno
boolean       INFRA_PIN_State = 0;      //Bit udávající stav snímače infra závory, 0 = neaktivováno, 1 = aktivováno - přerušen paprsek
boolean       INFRA_Stisk = 0;          //Bit udávající prvotní aktivaci infra snímače, 0 = neaktivován, 1 = aktivován
boolean       LED1_State = 0;           //Bit udávající stav výstupu na LED1, 0 = LED nesvítí, 1 = LED svítí
boolean       Cas_Tisk_Off_Set = 0;     //Bit udávající aktivaci časovky pro zpožděné vypnutí relé, 0 = není aktivována časovka
char          LED1_Function = 0;        //Proměnná udávající funkci signalizace zařízení pomocí LED1
char          Jumper_State = 0;         //Proměnná udávající nastavení jumperů J1 a J2 pro nastavení délky časovky zpožděného vypnutí relé
unsigned long CAS_TISK_OFF = 0;         //Proměnná výsledného nastaveného času pro vypnutí
unsigned long Cas_Tisk_Off = 0;         //Proměnná pro aktuální délku času pro vypnutí - deaktivace relé
unsigned long Cas_Stisku_tlac = 0;      //Proměnná pro určení délky stisku tlačítka
unsigned long Cas_Blik_Led1 = 0;        //Proměnná pro určení délky blikání LED1
unsigned long Cas_Aktiv_infra = 0;      //Proměnná pro určení délky aktivace infra snímače


//-----------------------------------------------------------------------------------------
//***   RESET LOOP - Část programu vykonávající se po resetu procesoru                  ***
//-----------------------------------------------------------------------------------------
void setup()
{
  pinMode(INFRA_PIN, INPUT);                //Inicializace vstupního pinu pro snímač - infrazávora
  pinMode(SWITCH_PIN, INPUT_PULLUP);        //Inicializace vstupního pinu pro tlačítko
  pinMode(J1_PIN, INPUT_PULLUP);            //Inicializace vstupního pinu pro jumper J1
  pinMode(J2_PIN, INPUT_PULLUP);            //Inicializace vstupního pinu pro jumper J2 
  pinMode(RELE_PIN, OUTPUT);                //Inicializace výstupního pinu pro relé
  pinMode(LED1_PIN, OUTPUT);                //Inicializace výstupního pinu pro LED1
  digitalWrite(RELE_PIN, RELE_AKTIV);       //Zapnutí výstupu na relé
  digitalWrite(LED1_PIN, LED1_AKTIV);       //Rozsvícení LED1 - indikace přítomnosti napájení
  LED1_State = 1;                           //Nastavení stavu LED pro rozsvícení
  SET_TIME_OFF();                           //Volání podprogramu pro nastavení časovky dle vstupních propojek J1 a J2
  Jumper_State = (Jumper_State * 2);        //Vynásobení čísla pro indikaci nastavené časovky pomocí blikání LED
  for (Jumper_State > 2; Jumper_State > 0; Jumper_State--)
    {
      LED1_State = !LED1_State;             //Neguj stav pro indikaci
      digitalWrite(LED1_PIN, LED1_State);   //Negovani stavu LED - blikani
      delay(250);
    }
  Cas_Tisk_Off_Set = 0;                     //Deaktivace časovky pro vypnutí
  INFRA_Stisk = 0;                          //Deaktivace pomocného bitu
  LED1_Function = 4;                        //Nepravidelné blikání LED1 pro indikaci přítomnosti napájení
}


//-----------------------------------------------------------------------------------------
//***   MAIN LOOP - Část programu vykonávající se neustále - nekonečná smyčka           ***
//-----------------------------------------------------------------------------------------
void loop()
{
  TLACITKO();                         //Volaní podprogramu pro zjištění stisku tlačítka
  INFRA_SNIMAC();                     //Volaní podprogramu pro zjištění stavu snímače infra závory
  LED1_STAV();                        //Volaní podprogramu pro indikaci stavu zařízení pomocí LED1
  RELE_Control();                     //Volání podprogramu pro ovládání relé
  Millis_error();                     //Volání podprogramu pro ošetření přetečení registru u funkce millis
}


//-----------------------------------------------------------------------------------------
//***   SET TIME OFF - Podprogram pro nastavení času vypnutí dle nastavení jumperů      ***
//-----------------------------------------------------------------------------------------
void SET_TIME_OFF(void)
{
  //Nastavení časovky dle vstupních jumperu J1 a J2
  J1_PIN_State = digitalRead(J1_PIN);       //Čtení stavu pinu na kterém je jumper J1
  J2_PIN_State = digitalRead(J2_PIN);       //Čtení stavu pinu na kterém je jumper J2
  
  if (J1_PIN_State == !J1_PIN_AKTIV)        //Je-li jumper J1 nepropojen-neaktivní, pokracuj      
    {
      if (J2_PIN_State == !J2_PIN_AKTIV)    //Je-li jumper J2 nepropojen-neaktivní, pokracuj
        {
          CAS_TISK_OFF = CAS_20min;         //Nastavení časovky 20min pro vypnutí
          Jumper_State = 4;                 //Nastavení proměnné pro indikaci nastavené časovky     
        }
        else                                //Je-li jumper J2 propojen-aktivní, pokracuj
        {
          CAS_TISK_OFF = CAS_15min;         //Nastavení časovky 15min pro vypnutí
          Jumper_State = 3;                 //Nastavení proměnné pro indikaci nastavené časovky
        }
    }
  if (J1_PIN_State == J1_PIN_AKTIV)         //Je-li jumper J1 propojen-aktivní, pokracuj     
    {
      if (J2_PIN_State == !J2_PIN_AKTIV)    //Je-li jumper J2 nepropojen-neaktivní, pokracuj
        {
          CAS_TISK_OFF = CAS_10min;          //Nastavení časovky 10min pro vypnutí
          Jumper_State = 2;                 //Nastavení proměnné pro indikaci nastavené časovky      
        }
        else                                //Je-li jumper J2 propojen-aktivní, pokracuj
        {
          CAS_TISK_OFF = CAS_5min;          //Nastavení časovky 5min pro vypnutí
          Jumper_State = 1;                 //Nastavení proměnné pro indikaci nastavené časovky
        }
    }
}


//-----------------------------------------------------------------------------------------
//***   RELE CONTROL - Podprogram pro ovládání rele                                     ***
//-----------------------------------------------------------------------------------------
void RELE_Control()
{

 if (Cas_Tisk_Off < millis() && Cas_Tisk_Off_Set == 1)    //Doběhla-li časovka vypnutí, proveď
      {
        digitalWrite(RELE_PIN, !RELE_AKTIV);              //Vypnutí výstupu na relé
        Cas_Tisk_Off_Set = 0;                             //Bit udávající deaktivaci časovky pro zpožděné vypnutí relé
        LED1_Function = 2;                                //Rychlé blikání LED1 - indikace doběhlé časovky a vypnutí relé
      }   
  
}


//-----------------------------------------------------------------------------------------
//***   TLACITKO - Podprogram pro zjištění stisku tlačítka                              ***
//-----------------------------------------------------------------------------------------
void TLACITKO()
{
  SWITCH_PIN_State = digitalRead(SWITCH_PIN);               //Čtení stavu pinu na kterém je tlačítko
  if (SWITCH_PIN_State == SWITCH_AKTIV)                     //Je-li tlačítko stisknuto (vstup v aktivním stavu)
      {
        LED1_Function = 0;                                  //Zhasnutí LED1 - indikace stisku tlačítka
        LED1_STAV();                                        //Volání podprogramu pro indikaci stavu zařízení pomocí LED1
        SWITCH_Stisk = 1;                                   //Nastavení bitu, protože došlo ke stisku tlačítka
        Cas_Stisku_tlac = millis();                         //Zapamatování času při vyvolání stisku tlačítka
      }

   if (SWITCH_Stisk == 1)                                   //Je-li stisknuto tlačítko, proveď
  {
      do{
          if (millis() > Cas_Stisku_tlac + CAS_STISK_LONG)  //Je-li stisk tlačítka větší než je nastavený čas, proveď
              {
                digitalWrite(RELE_PIN, !RELE_AKTIV);        //Vypnutí výstupu na relé
                INFRA_Stisk = 1;                            //Aktivace pomocného bitu pro aktivaci infra snímače
                LED1_Function = 2;                          //Rychlé blikání LED1 - indikace doběhlé časovky a vypnutí relé            
              } 
        }  while(digitalRead(SWITCH_PIN) == SWITCH_AKTIV);  //Je-li stále stisknuto tlačítko, čekej a počítej čas.
 
     if (millis() < Cas_Stisku_tlac + CAS_STISK_LONG)       //Je-li stisk tlačítka menší než je nastevený čas, proveď
              {
                setup();                                    //Resetování programu - běh od počátku  
              } 
    SWITCH_Stisk = 0;                                       //Vynuluj pomocný bit stisku tlačítka a čekej na další stisk 
  }
}


//-----------------------------------------------------------------------------------------
//***   INFRA ZAVORA - Podprogram pro zjištění stavu snímače infra závory               ***
//-----------------------------------------------------------------------------------------
void INFRA_SNIMAC()
{
  INFRA_PIN_State = digitalRead(INFRA_PIN);                 //Čtení stavu pinu na kterém je snímač - infra závora
  if (INFRA_PIN_State== INFRA_AKTIV && INFRA_Stisk == 0)    //Je-li infra závora aktivována - čekej a potom aktivuj časovku
    {
      Cas_Aktiv_infra = millis();                           //Zapamatuj si čas aktivace infra snímače
      INFRA_Stisk = 1;                                      //Aktivace pomocného bitu pro aktivaci infra snímače
      LED1_Function = 1;                                    //Rozsvícení LED1 - indikace aktivace INFRA závory
    }
   
  if (INFRA_PIN_State== !INFRA_AKTIV && Cas_Tisk_Off_Set== 0 && digitalRead(RELE_PIN)== RELE_AKTIV)   //Je-li infra závora deaktivována a byla provedena aktivace
    {
      LED1_Function = 4;                                    //Nepravidelné blikání LED1 pro indikaci přítomnosti napájení
      INFRA_Stisk = 0;                                      //Deaktivace pomocného bitu pro aktivaci infra snímače
    }


if (millis() > Cas_Aktiv_infra + CAS_STISK_LONG && digitalRead(RELE_PIN)== RELE_AKTIV)    //Je-li délka aktivace infra snímače větší než je nastavený čas, proveď
      { 
        if (INFRA_PIN_State== INFRA_AKTIV && Cas_Tisk_Off_Set== 0)//Je-li infra závora aktivována - přerušen paprsek a není-li nastaven čas vypnutí
          {
            Cas_Tisk_Off = millis() + CAS_TISK_OFF;         //Nastav časovku pro zpožděné vypnutí relé
            Cas_Tisk_Off_Set = 1;                           //Bit udávající aktivaci časovky pro zpožděné vypnutí relé
            LED1_Function = 3;                              //Pomalé blikání LED1 - indikace aktivace INFRA a časovky pro zpožděné vypnutí
          }
        if (INFRA_PIN_State== !INFRA_AKTIV && Cas_Tisk_Off_Set== 0)//Není-li infra závora aktivována - nepřerušen paprsek a není-li nastaven čas vypnutí
          {
            INFRA_Stisk = 0;                                //Deaktivace pomocného bitu pro prvotní aktivaci od infra závory
            LED1_Function = 4;                              //Nepravidelné blikání LED1 pro indikaci přítomnosti napájení
          }
      }
}


//-----------------------------------------------------------------------------------------
//***   LED1 - Podprogram pro indikaci stavu zařízení                                   ***
//-----------------------------------------------------------------------------------------
void LED1_STAV(void)
{
  switch (LED1_Function)
    {
    case 0:   //Zhasnutí LED1
              digitalWrite(LED1_PIN, !LED1_AKTIV);             //Zhasnutí LED1 
              break;
    case 1:   //Rozsvícení LED1
              digitalWrite(LED1_PIN, LED1_AKTIV);              //Rozsvícení LED1
              break;
    case 2:   //Rychlé blikání LED1 perioda 100ms a střída 50%
              if (Cas_Blik_Led1 < millis())                    //Doběhla-li časovka pro rozsvícení LED
                {
                  LED1_State = !LED1_State;                   //Neguj stav pro indikaci
                  digitalWrite(LED1_PIN, LED1_State);         //Negovani stavu LED - blikani
                  Cas_Blik_Led1 = millis() + CAS_BLIK1_LED1;  //Nastav časovku pro zhasnuti LED1
                }  
              break;
    case 3:   //Pomalé blikání LED1 perioda 2000ms a střída 50%
              if (Cas_Blik_Led1 < millis())                   //Doběhla-li časovka pro rozsvícení LED
                {
                  LED1_State = !LED1_State;                   //Neguj stav pro indikaci
                  digitalWrite(LED1_PIN, LED1_State);         //Negovani stavu LED - blikani
                  Cas_Blik_Led1 = millis() + CAS_BLIK2_LED1;  //Nastav časovku pro zhasnuti LED1
                }  
              break;  
    case 4:   //Blikání LED1 nepravidelné perioda 2000ms a pouze 50ms svit
              if (Cas_Blik_Led1 < millis())                   //Přečti časový údaj z procesoru a zkontroluj nastaveny cas
                { 
                  LED1_State = !LED1_State;                   //Neguj stav pro indikaci
                  digitalWrite(LED1_PIN, LED1_State);         //Negovani stavu LED - blikani
                  switch (LED1_State)                         //Rozdílné délky času pro nepravidelné bliknutí LED1
                      {
                        case 0: Cas_Blik_Led1 = millis() + CAS_BLIK2_LED1;     //Nastavení délky pro zhasnutí LED
                                if (LED1_AKTIV == 0)          //Je-li aktivace LED1 opačná, nastav
                                   {
                                     Cas_Blik_Led1 = millis() + CAS_BLIK1_LED1;//Nastavení délky pro zhasnutí LED
                                   }
                                break;        
                        case 1: Cas_Blik_Led1 = millis() + CAS_BLIK1_LED1;     //Nastavení délky pro rozsvícení LED
                                if (LED1_AKTIV == 0)          //Je-li aktivace LED1 opačná, nastav
                                   {
                                     Cas_Blik_Led1 = millis() + CAS_BLIK2_LED1;//Nastavení délky pro zhasnutí LED
                                   }
                                break;
                      };  //Konec switch LED1_State - nastavení rozdílné délky času
                }                                             //Konec indikace stavu pomoci blikani LED
              break;    
      }; //konec switch LED1_Function
}


//-----------------------------------------------------------------------------------------
//***   Funkce Millis - oprava funkce přetečení long integeru                           ***
//***   long integer je 2 na 32 = 4294967295ms = 49,71dní, potom čítá opět od nuly      ***
//-----------------------------------------------------------------------------------------
void Millis_error(void)
{
  if( millis() > 4294967000 )           //Blíží-li se čas přetečení registru pro funkci millis, proveď vynulování časovek
    {
      SET_TIME_OFF();                   //Volání podprogramu pro nastavení časovky dle vstupních jumperu J1 a J2
      Cas_Tisk_Off = 0;                 //Vynulování časovky
      Cas_Blik_Led1 = 0;                //Vynulování časovky
      Cas_Stisku_tlac = 0;              //Vynulování časovky
      Cas_Aktiv_infra = 0;              //Vynulování časovky
    }
}
